/***************************************************************************
 *   Copyright (C) 2007 by Helio Chissini de Castro   *
 *   helio@mandriva.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include "kconfigldap.h"

#include <qdragobject.h>

#include <kaction.h>
#include <kconfigdialog.h>
#include <kglobal.h>
#include <klocale.h>
#include <kprinter.h>
#include <kstatusbar.h>
#include <kiconloader.h>

kconfigldap::kconfigldap()
		: KMainWindow ( 0, "kconfigldap" ),
		m_view ( new kconfigldapView ( this ) ),
		m_printer ( 0 )
{
	// tell the KMainWindow that this is indeed the main widget
	setCentralWidget ( m_view );
	// and a status bar
	statusBar()->show();
	// then, setup our actions
	setupActions();

	// apply the saved mainwindow settings, if any, and ask the mainwindow
	// to automatically save settings if changed: window size, toolbar
	// position, icon size, etc.
	setAutoSaveSettings();

	// allow the view to change the statusbar and caption
	connect ( m_view, SIGNAL ( signalChangeStatusbar ( const QString& ) ),
	          this,   SLOT ( changeStatusbar ( const QString& ) ) );
	connect ( m_view, SIGNAL ( signalChangeCaption ( const QString& ) ),
	          this,   SLOT ( changeCaption ( const QString& ) ) );

}

kconfigldap::~kconfigldap()
{}

void kconfigldap::setupActions()
{
	KStdAction::quit ( kapp, SLOT ( quit() ), actionCollection() );
	//	KStdAction::preferences ( this, SLOT ( optionsPreferences() ), actionCollection() );

	// custom menu and menu item
	new KAction ( i18n ( "Dump Config from Application" ),
	              QIconSet ( BarIcon ( "dumpldap" ) ), 0,
	              m_view, SLOT ( dumpConfig() ),
	              actionCollection(), "dump_action" );

	new KAction ( i18n ( "Export LDIF Resources" ),
	              QIconSet ( BarIcon ( "ldifldap" ) ), 0,
	              m_view, SLOT ( exportLDIF() ),
	              actionCollection(), "ldif_action" );
	setupGUI();
}

void kconfigldap::newToolbarConfig()
{
	// this slot is called when user clicks "Ok" or "Apply" in the toolbar editor.
	// recreate our GUI, and re-apply the settings (e.g. "text under icons", etc.)
	setupGUI();
}

void kconfigldap::optionsPreferences()
{
	// The preference dialog is derived from prefs-base.ui which is subclassed into Prefs
	//
	// compare the names of the widgets in the .ui file
	// to the names of the variables in the .kcfg file
	//KConfigDialog *dialog = new KConfigDialog ( this, "settings", Settings::self(), KDialogBase::Swallow );
	//dialog->addPage ( new Prefs(), i18n ( "General" ), "package_settings" );
	//connect ( dialog, SIGNAL ( settingsChanged() ), m_view, SLOT ( settingsChanged() ) );
	//dialog->show();
}

void kconfigldap::changeCaption ( const QString& text )
{
	// display the text on the caption
	setCaption ( text );
}

void kconfigldap::changeStatusbar ( const QString& text )
{
	// display the text on the statusbar
	statusBar()->message ( text );
}

#include "kconfigldap.moc"

