/***************************************************************************
 *   Copyright (C) 2007 by Helio Chissini de Castro   helio@mandriva.com   *
 *   Copyright (C) 2007 by Gustavo Boiko              boiko@mandriva.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <kprocess.h>
#include <kstandarddirs.h>
#include <klistview.h>
#include "klistviewsearchline.h"

#include <qapplication.h>
#include <qlistview.h>
#include <qheader.h>
#include <qdom.h>
#include <qdir.h>
#include <qlayout.h>

#include "kconfigldapappdump.h"
#include "kconfigldapview.h"

kconfigldapAppDump::kconfigldapAppDump ( QWidget* parent, const char* name, bool modal, WFlags fl )
		: kconfigldapappdump_base ( parent,name, modal,fl )
{
	m_parent = (kconfigldapView*)parent;
	
	load ( KServiceGroup::root() );
	setupWidgets();
}

kconfigldapAppDump::~kconfigldapAppDump()
{}

/*!
    \fn kconfigldapAppDump::kdeApps()
 */
QStringList kconfigldapAppDump::kdeApps() const
{
	return m_KDEApps.keys();
}


/*!
    \fn kconfigldapAppDump::nonkdeApps()
 */
QStringList kconfigldapAppDump::nonkdeApps() const
{
	return m_NonKDEApps;
}

	void kconfigldapAppDump::load ( KServiceGroup::Ptr group )
{
	if ( !group || !group->isValid() )
		return;

	KServiceGroup::List list = group->entries ( true, true, true, false );

	KServiceGroup::List::ConstIterator it;
	for ( it = list.begin(); it != list.end(); it++ )
	{
		KSycocaEntry *s = ( *it );
		if ( s->isType ( KST_KServiceGroup ) )
		{
			KServiceGroup *group = static_cast<KServiceGroup*> ( s );
			load ( group );
		}
		if ( s->isType ( KST_KService ) )
		{
			KService *service = static_cast<KService*> ( s );
			if ( service->menuId().startsWith ( "kde-" ) )
			{
				m_KDEApps.insert( service->name(), service->exec() );
			}
			else
				m_NonKDEApps.append ( service->name() );
		}
	}
}

void kconfigldapAppDump::accept()
{
	KProcess *proc = new KProcess;
	QListViewItem *it = appList->selectedItem();
	if( it == 0 )
	{
		QDialog::reject();
		return;
	}

	proc->setEnvironment( "DUMPCONFIG", locateLocal("appdata", "xmldump/", true) );
	proc->setEnvironment( "KCONFIGXMLDIR", locateLocal("appdata", "xmldump/", true) );

	QStringList app = QStringList::split( " ", m_KDEApps[it->text(0)] );
	
	for ( QStringList::Iterator it = app.begin(); it != app.end(); ++it ) 
		*proc << *it;

	proc->start( KProcess::Block );
	processDone( it->text(0) );
	
	QDialog::accept();
}

/*!
    \fn kconfigldapAppDump::setupWidgets()
 */
void kconfigldapAppDump::setupWidgets()
{
	QGridLayout *grid = (QGridLayout*)layout();
	grid->addMultiCellWidget( new KListViewSearchLineWidget( appList, this ), 0, 0, 0, 1 );
	
	appList->header()->setStretchEnabled ( true, -1 );
	appList->header()->adjustHeaderSize();
	// Add apps from kservicegroup
	QStringList list = kdeApps();
	QStringList::iterator it;
	for ( it = list.begin(); it != list.end(); ++it )
		new QListViewItem ( appList, ( *it ).latin1() );
}

/*!
    \fn kconfigldapAppDump::processDone()
 */
void kconfigldapAppDump::processDone( const QString& app )
{
	QDir dir( locateLocal("appdata", "xmldump/" ) );
	if ( !dir.exists() )
		return;

	int appid = m_parent->getDb()->addApplication( app );
	m_parent->updateApplicationListView();
	
	QStringList files = dir.entryList( QDir::Files );
	QStringList::iterator it;
	for( it = files.begin(); it != files.end(); ++it )
	{
		QDomDocument doc;
		QFile f( dir.filePath( *it ) );
		if ( f.open( IO_ReadOnly ) )
		if ( doc.setContent( &f ) )
		{
			QDomElement docElem = doc.documentElement();
			int resid = m_parent->getDb()->addResource( docElem.tagName() );
			m_parent->getDb()->addAppRes( appid, resid );
			QDomNode node = docElem.firstChild();
			while ( ! node.isNull() )
			{
				QDomNode grpElem = node.toElement();
				if ( ! grpElem.isNull() )
				{
					int grpid = m_parent->getDb()->addGroup( resid, grpElem.attributes().namedItem( "name" ).nodeValue() );
					QDomNode entryNode = grpElem.firstChild();
					while( ! entryNode.isNull() )
					{
						m_parent->getDb()->addEntry(
							grpid,
							m_parent->getTypeId( entryNode.attributes().namedItem( "type" ).nodeValue() ),
														entryNode.attributes().namedItem( "name" ).nodeValue(),
														entryNode.toElement().text() );
						entryNode = entryNode.nextSibling();
					}
				}
				node = node.nextSibling();
			}
		}
		f.close();
		f.remove();
	}
}


#include "kconfigldapappdump.moc"
