/***************************************************************************
 *   Copyright (C) 2007 by Helio Chissini de Castro   *
 *   helio@mandriva.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "kconfigldapdatabase.h"

#include <qstringlist.h>
#include <qsqldatabase.h>
#include <qsqlquery.h>
#include <kstandarddirs.h>

kconfigldapDatabase::kconfigldapDatabase()
{
	setupDB();
}


kconfigldapDatabase::~kconfigldapDatabase()
{}

/*!
    \fn kconfigldapDatabase::setupDB()
 */
void kconfigldapDatabase::setupDB()
{
	KStandardDirs stddir;
	m_sqlitedb = QSqlDatabase::addDatabase ( "QSQLITE" );
	m_sqlitedb->setDatabaseName ( locateLocal("appdata", "database/", true) + "kconfigldapDB" );
	
	if ( !m_sqlitedb->open() )
	{
		m_sqlitedb->lastError().showMessage ( "An error occured. Please read the README file in the sqltable"
		                                      "dir for more information.\n\n" );
		return;
	}

	QSqlQuery query ( "CREATE TABLE applications ( id INTEGER PRIMARY KEY,app TEXT UNIQUE )", m_sqlitedb );
	query.exec ( "CREATE TABLE resources ( id INTEGER PRIMARY KEY, res TEXT UNIQUE )" );
	query.exec ( "CREATE TABLE appres( id INTEGER PRIMARY KEY, app INTEGER, res INTEGER )" );
	query.exec ( "CREATE TABLE groups ( id INTEGER PRIMARY KEY, res INTEGER, grpname TEXT )" );
	query.exec ( "CREATE TABLE data ( id INTEGER PRIMARY KEY, grp INTEGER, type INTEGER, entryname TEXT, entryvalue TEXT )" );
	m_sqlitedb->close();
}

/*!
    \fn kconfigldapDatabase::getApplications()
 */
QStringList kconfigldapDatabase::getApplications() const
{
	QStringList appList;

	if ( m_sqlitedb->open() )
	{
		QSqlQuery query ( "SELECT app FROM applications;", m_sqlitedb );
		if ( query.isActive() )
		{
			while ( query.next() )
			{
				appList += query.value ( 0 ).toString();
			}
		}
		m_sqlitedb->close();
	}

	return appList;
}

/*!
    \fn kconfigldapDatabase::getAllResources()
 */
QStringList kconfigldapDatabase::getAllResources() const
{
	QStringList resList;

	if ( m_sqlitedb->open() )
	{
		QSqlQuery query ( "SELECT * FROM resources;", m_sqlitedb );
		if ( query.isActive() )
		{
			while ( query.next() )
			{
				resList += query.value ( 1 ).toString();
			}
		}
		m_sqlitedb->close();
	}

	return resList;
}

/*!
    \fn kconfigldapDatabase::getAppResource( QString app )
 */
QStringList kconfigldapDatabase::getAppResources( const QString& app ) const
{
	QStringList resList;

	if ( m_sqlitedb->open() )
	{
		QSqlQuery query ( "SELECT id FROM applications WHERE app='" + app +"'",
								m_sqlitedb );
		QSqlQuery target;
		if ( query.isActive() )
		{
			query.next();
			query.exec( "SELECT res FROM appres WHERE app=" + query.value( 0 ).toString() );
			while ( query.next() )
			{
				target.exec( "SELECT res FROM resources WHERE id=" + query.value( 0 ).toString() );
				target.next();
				resList += target.value ( 0 ).toString();
			}
		}
		m_sqlitedb->close();
	}

	return resList;
}

/*!
    \fn kconfigldapDatabase::getGroups( QString res )
 */
QStringList kconfigldapDatabase::getGroups ( const QString& res ) const
{
	QStringList grpList;
	
	if ( m_sqlitedb->open() )
	{
		QSqlQuery query( "SELECT id FROM resources WHERE res='" + res + "'" );
		QSqlQuery target;
		query.next();

		target.exec( " SELECT grpname FROM groups WHERE res=" + query.value ( 0 ).toString() );
		if ( target.isActive() )
		{
			while ( target.next() )
			{
				grpList += target.value ( 0 ).toString();
			}
		}
		m_sqlitedb->close();
	}
	return grpList;
}


/*!
    \fn kconfigldapDatabase::getGroupData( QString res, QString grp )
 */
GroupDataEntries kconfigldapDatabase::getGroupData( const QString& res, const QString& grp ) const
{
	GroupDataEntries dataEntries;
	if ( m_sqlitedb->open() )
	{
		QSqlQuery query( "SELECT a.* from data a, resources b, groups c WHERE ( b.res='" +
				res +
				"' ) and ( c.res = b.id ) and ( c.grpname='" +
				grp +
				"') and (a.grp = c.id)" );
		if ( query.isActive() )
		{
			while ( query.next() )
			{
				dataEntries.push_back( GroupData(
									  query.value( 2 ).toInt(),
									  query.value( 3 ).toString(),
									  query.value( 4 ).toString() ) );
			}
		}
		m_sqlitedb->close();
	}
	return dataEntries;
}

/*!
    \fn kconfigldapDatabase::setGroupData( const QString& res, const QString& grp, const QString& entry, const QString& data )
 */
void kconfigldapDatabase::setGroupData( const QString& res, const QString& grp, const QString& entry, const QString& data ) const
{
	if ( m_sqlitedb->open() )
	{
		QSqlQuery query( QString( "SELECT id from resources WHERE res = '%1'").arg( res ) );
		query.next();
		int resid = query.value( 0 ).toInt();
		query.exec( QString( "SELECT id from groups WHERE res=%1 AND grpname='%2'").arg( resid ).arg( grp ) );
		query.next();
		int grpid = query.value( 0 ).toInt();
		
		QString updQuery = QString( "UPDATE data SET entryvalue='%1' WHERE grp=%2 AND entryname='%3'" ).arg( data ).arg( grpid ).arg( entry );
		query.exec( updQuery );
	}
	
}

/*!
    \fn kconfigldapDatabase::addApplication( const QString& app ) const
 */
int kconfigldapDatabase::addApplication( const QString& app ) const
{
	if ( m_sqlitedb->open() )
	{
		QSqlQuery query( "INSERT INTO applications ( app ) VALUES ( '" + app +  "')" );
		query.exec( "SELECT id FROM applications WHERE app='" + app + "'" );
		if ( query.isActive() )
		{
			query.next();
			return query.value(0).toInt();
		}
	}
	return -1;
}


/*!
    \fn kconfigldapDatabase::addResource( const QString& res ) const
 */
int kconfigldapDatabase::addResource( const QString& res ) const
{
	if ( m_sqlitedb->open() )
	{
		QSqlQuery query( "INSERT INTO resources ( res ) VALUES ( '" + res +  "')" );
		query.exec( "SELECT id FROM resources WHERE res='" + res + "'" );
		if ( query.isActive() )
		{
			query.next();
			return query.value(0).toInt();
		}
	}
	return -1;
}


/*!
    \fn kconfigldapDatabase::addAppRes( int app, int res )
 */
void kconfigldapDatabase::addAppRes( int app, int res )
{
	if ( m_sqlitedb->open() )
	{
		QSqlQuery query( "SELECT id FROM appres WHERE app=" +
				QString::number( app ) +
				" AND res=" +
				QString::number( res ) );
		if( query.isActive() )
			if( query.next() )
				return;
		
		query.exec( "INSERT INTO appres ( app, res ) VALUES ( " +
				QString::number( app ) +  ", " +
				QString::number( res ) + " )" );
	}
}

/*!
    \fn kconfigldapDatabase::addGroup( int res, const QString& grp ) const
 */
int kconfigldapDatabase::addGroup( int res, const QString& grp ) const
{
	if ( m_sqlitedb->open() )
	{
		QSqlQuery query( "SELECT id FROM groups WHERE res=" +
				QString::number( res ) +
				" AND grpname='" +
				grp + "'" );
		if( query.isActive() )
			if( query.next() )
				return query.value( 0 ).toInt();
		
		query.exec( "INSERT INTO groups ( res, grpname ) VALUES ( " +
				QString::number( res ) +  ", '" +
				grp + "' )" );

		query.exec( "SELECT id FROM groups WHERE res=" +
				QString::number( res ) +
				" AND grpname='" +
				grp + "'" );
		if( query.isActive() )
			if( query.next() )
				return query.value( 0 ).toInt();
	}
	return -1;
}


/*!
    \fn kconfigldapDatabase::addEntry( int grpid, int type, const QString& name, const QString& entryval )
 */
void kconfigldapDatabase::addEntry( int grpid, int type, const QString& name, const QString& entryval )
{
	if ( m_sqlitedb->open() )
	{
		QSqlQuery query( QString("SELECT * FROM data where grp=%1 and entryname='%2'").arg( grpid ).arg( name ) );
		if( query.isActive() )
			if( query.next() )
				return;

		query.exec( "INSERT INTO data ( grp, type, entryname, entryvalue ) VALUES ( " +
				QString::number( grpid ) + "," +
				QString::number( type ) + ",'" +
				name + "','" +
				entryval + "' )" );
	}
}
