/***************************************************************************
 *   Copyright (C) 2007 by Helio Chissini de Castro   *
 *   helio@mandriva.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include "qcolortableitem.h"
#include "qfonttableitem.h"
#include "kconfigldapview.h"
#include "kconfigldapappdump.h"
#include "kconfigldapldif.h"

#include <qlabel.h>
#include <qlistview.h>
#include <qheader.h>
#include <qtable.h>
#include <qfile.h>
#include <qmessagebox.h>
#include <klocale.h>
#include <kdirselectdialog.h>
#include <kstandarddirs.h>
#include <kcombobox.h>

kconfigldapView::kconfigldapView ( QWidget *parent )
		: kconfigldapview_base ( parent )
{
	//settingsChanged();

	// Create new Db object
	m_db = new kconfigldapDatabase();

	setupWidgets();

}

kconfigldapView::~kconfigldapView()
{}

void kconfigldapView::dumpConfig()
{
	appList->clearSelection();
	resourceList->clear();

	kconfigldapAppDump dlg ( this );
	dlg.exec();
}

/*!
    \fn kconfigldapView::exportLDIF()
 */
void kconfigldapView::exportLDIF()
{
	KDirSelectDialog *dirdlg = new KDirSelectDialog (
	                               locateLocal ( "appdata", "LDIF/", true ) );

	if( dirdlg->exec() != QDialog::Accepted )
		return;
	
	if( ! dirdlg->url().isLocalFile() )
		return;

	QFile file( dirdlg->url().path() + "/kconfigldap.ldif" );

	if ( file.open( IO_WriteOnly ) )
	{
		//Call ldif export dialog
		kconfigldapLdif ldifdlg( this );

		
		if( ldifdlg.exec() == QDialog::Rejected )
			return;

		QString fullbase = ldifdlg.ldifSuffixResult->text();
		QTextStream out ( &file );
		out.setEncoding ( QTextStream::UnicodeUTF8 );

		out << "# extended LDIF" << endl;
		out << "# LDAPv3" << endl;
		out << "# base <" << fullbase << "> with scope sub" << endl;
		out << "# filter: ( objectclass=* )" << endl;
		out << "# requesting: ALL" << endl << "#" << endl << endl;
		out << "#dn: " << fullbase << endl;
		out << "#objectClass: organizationalUnit"<< endl;
		out << "#ou: default" << endl;
		out << "#description: Configuration entries to play with in KDE" << endl << endl;

		// Output main config entry - appConfig

		QListViewItemIterator it( ldifdlg.ldifList->firstChild() );
		while( it.current() )
		{
			QString value = it.current()->text( 0 ) + "  " + QString::number( ((QCheckListItem*)it.current())->state() );

			if( it.current()->depth() != 1 || ((QCheckListItem*)it.current())->state() == QCheckListItem::Off )
			{
				++it;
				continue;
			}
			QString cfgbase = "cn=" + it.current()->text( 0 ) + "," + fullbase;
			out << "# " << it.current()->text( 0 ) << " KDEConfig" << endl;
			out << "dn: " << cfgbase << endl;
			out << "objectClass: appConfig" << endl << "cn: " << it.current()->text( 0 ) << endl << endl;
		
			// Output sections - appConfigSection
			QListViewItemIterator entit( it.current()->firstChild(), QListViewItemIterator::Checked );
			while( entit.current() )
			{
				QCheckListItem *grp = (QCheckListItem*)entit.current();
				if( grp )
				{
					if( grp->isOn() && grp->depth() == 2 )
					{
						QString test = grp->text( 0 );
						out << "# Group " << grp->text( 0 ) << " , "  << it.current()->text( 0 ) << " KDEConfig" << endl;
						out << "dn: cn=" << grp->text( 0 ) << "," << cfgbase << endl;
						out << "appname: " << it.current()->text( 0 ) << endl;
						out << "cn: " << grp->text( 0 ) << endl;
						out << "objectClass: appConfigSection" << endl;
						if( ((QCheckListItem*)grp->firstChild())->isOn() )
							out << "immutable: TRUE" << endl;
						else
							out << "immutable: FALSE" << endl;
					}

					GroupDataEntries dataEntries = m_db->getGroupData ( it.current()->text( 0 ), grp->text ( 0 ) );
					GroupDataEntries::iterator datait;
					for ( datait = dataEntries.begin(); datait != dataEntries.end(); ++datait )
						out << "appconfigentry: " <<
								(*datait).name() << "=" <<
								(*datait).value() << endl;
				}
				out << endl;
				++entit;
			}
			out << endl;
			++it;
		}
		
		file.close();
	}
}


/*!
    \fn kconfigldapView::setupWidgets()
 */
void kconfigldapView::setupWidgets()
{
	// Applications list
	appList->header()->setStretchEnabled ( true, -1 );
	appList->header()->adjustHeaderSize();
	updateApplicationListView();


	// Resources list
	resourceList->header()->setStretchEnabled ( true, -1 );
	resourceList->header()->adjustHeaderSize();

	// Item table
	itemTable->horizontalHeader()->setStretchEnabled ( true, 2 );
	itemTable->horizontalHeader()->adjustHeaderSize();
	itemTable->verticalHeader()->hide();
}


/*!
	\fn kconfigldapView::updateApplicationListView()
 */
void kconfigldapView::updateApplicationListView()
{
	appList->clear();

	new QListViewItem ( appList, i18n ( "All Application Resources" ) );

	// Add apps from database
	QStringList list = m_db->getApplications();
	QStringList::iterator it;
	for ( it = list.begin(); it != list.end(); ++it )
		new QListViewItem ( appList, ( *it ).latin1() );
}

/*!
	\fn kconfigldapView::updateResourceListView( QListViewItem *item )
 */
void kconfigldapView::updateResourceListView ( QListViewItem *item )
{
	QString sel = item->text ( 0 );
	QListViewItem *itParent;
	QStringList::iterator it;
	QStringList list;

	resourceList->clear();

	if ( sel == i18n ( "All Application Resources" ) )
		list = m_db->getAllResources();
	else
		list = m_db->getAppResources ( sel );

	for ( it = list.begin(); it != list.end(); ++it )
	{
		itParent = new QListViewItem ( resourceList, ( *it ).latin1() );
		QStringList groups = m_db->getGroups ( *it );
		QStringList::iterator resit;
		for ( resit = groups.begin(); resit != groups.end(); ++resit )
			new QListViewItem ( itParent, ( *resit ).latin1() );
		itParent->setOpen( true );
	}

}

/*!
    \fn kconfigldapView::updateDataTable( QListViewItem *item )
 */
void kconfigldapView::updateDataTable ( QListViewItem *item )
{
	QListViewItem *parent = item->parent();
	if ( parent == 0 )
		return;

	// Set number of entrie in QTable to 0 to clean
	itemTable->setNumRows ( 0 );

	GroupDataEntries dataEntries = m_db->getGroupData ( parent->text ( 0 ), item->text ( 0 ) );
	m_currentGroup = item->text ( 0 );
	m_currentResource = parent->text ( 0 );
	GroupDataEntries::iterator it;
	int row = 0;

	for ( it = dataEntries.begin(); it != dataEntries.end(); ++it )
	{
		itemTable->insertRows ( row );
		itemTable->setText ( row, 0, ( *it ).name() );

		QFont font;
		QCheckTableItem *checkItem;

		switch ( ( *it ).type() )
		{
			case BOOLTYPE:
				itemTable->setText ( row, 1, "Bool" );
				checkItem = new QCheckTableItem ( itemTable, "Status" );
				checkItem->setChecked ( ( *it ).value().toInt() );
				itemTable->setItem ( row, 2, new QCheckTableItem ( itemTable, "check" ) );
				break;
			case COLORTYPE:
				itemTable->setText ( row, 1, "Color" );
				itemTable->setItem ( row, 2, new QColorTableItem ( itemTable, QTableItem::Always, ( *it ).value(), QColor ( ( *it ).value() ) ) );
				break;
			case LISTTYPE:
				itemTable->setText ( row, 1, "List" );
				itemTable->setItem ( row, 2, new QComboTableItem ( itemTable, QStringList::split ( ",", ( *it ).value() ),TRUE ) );
				break;
			case FONTTYPE:
				itemTable->setText ( row, 1, "Font" );
				font.fromString ( ( *it ).value() );
				itemTable->setItem ( row, 2, new QFontTableItem ( itemTable, QTableItem::Always, ( *it ).value(), font ) );
				break;
			case NUMTYPE:
				itemTable->setText ( row, 1, "Num" );
				itemTable->setText ( row, 2, ( *it ).value() );
				break;
			case DOUBLETYPE:
				itemTable->setText ( row, 1, "Double" );
				itemTable->setText ( row, 2, ( *it ).value() );
				break;
			case STRINGTYPE:
				itemTable->setText( row, 1, "String" );
				itemTable->setText( row, 2 , ( *it ).value() );
				break;
			case UNSIGNEDTYPE:
				itemTable->setText ( row, 1, "Unsigned" );
				itemTable->setText ( row, 2, ( *it ).value() );
				break;
		}

		row++;
	}	
}

/*!
    \fn kconfigldapView::getTypeId( const QString& type ) const
 */
ConfigTypes kconfigldapView::getTypeId ( const QString& type ) const
{
	if ( type == "String" )
		return STRINGTYPE;
	else if ( type == "Bool" )
		return BOOLTYPE;
	else if ( type == "Num" )
		return NUMTYPE;
	else if ( type == "Double" )
		return DOUBLETYPE;
	else if ( type == "List" )
		return LISTTYPE;
	else if ( type == "Color" )
		return COLORTYPE;
	else if ( type == "Font" )
		return FONTTYPE;
	else if ( type == "UnsignedNum" )
		return UNSIGNEDTYPE;
	else
		return INVALIDTYPE;
}

void kconfigldapView::updateItem( int row, int col )
{
	if( col != 2 )
		return;
	
	QString data;
	QString type =  itemTable->text( row, 1 );

	if ( type=="Bool" )
	{
		QCheckTableItem* it = (QCheckTableItem *)itemTable->item( row, 2 );
		data = it->isChecked() ? "1" : "0";
	}
	else if ( type == "List" )
	{
		QComboTableItem *it = (QComboTableItem *)itemTable->item( row, 2 );
		QStringList items;
		for( int i = 0; i < it->count(); i++ )
			items << it->text( i );
		data = items.join( "," );
	}
	else if ( type == "Color" )
	{
		QColorTableItem *it = (QColorTableItem *)itemTable->item( row, 2 );
		data = it->text();
	}
	else if ( type == "Font" )
	{
		QFontTableItem *it = (QFontTableItem *)itemTable->item( row, 2 );
		data = it->text();
	}
	else
		data = itemTable->text( row, 2 );
	
	m_db->setGroupData ( m_currentResource, m_currentGroup, itemTable->text( row, 0 ), data );
}

#include "kconfigldapview.moc"